/* Compute the eigen value decomposition of a 3 by 3 symetric matrix
Input:
A = [ A11  A12  A13     = [ A[0]  A[1]  A[2]
A21  A22  A23          0    A[3]  A[4]
A31  A32  A33 ];       0     0    A[5] ];
Output:
eigenvalues, eigenvectors

Reference: http://en.wikipedia.org/wiki/Eigenvalue_algorithm#3.C3.973_matrices
*/
// #include <cmath>
// #include <assert.h>
// #include <algorithm>
// #include <iostream>




// eigenvalue decomposition of a three by three symmetric matrix
template<typename T=float>
void eigen_decomp( const T A[6], T eigenvalues[3], T eigenvectors[3][3] );

// give a 3 by 3 symmetric matrix and its eigenvalues, compute the corresponding eigenvectors
template<typename T=float>
inline void eigenvector_from_value( const T A[6], const T& eig1, const T& eig2, T eigenvector[3] );

// normalize a vector
template<typename T=float>
inline void normalize( T v[3] );

// compute the cross product of two vectors
template<typename T=float>
inline void cross_product( const T v1[3], const T v2[3], T v3[3] );

// given a vector v1, compute two arbitrary normal vectors v2 and v3
template<typename T=float>
void normal_vectors(  const T v1[3], T v2[3], T v3[3] ) ;
