#include "data3D.h"
#include <opencv2/imgproc/imgproc.hpp>

// template<typename T> class Data3D;

template<typename T = float>
class Kernel3D : public Data3D<T>
{
    // operator overload
    template <typename U>
    friend std::ostream& operator<<(std::ostream& out, const Kernel3D<U>& data);

public:
    // constructor and destructors
    Kernel3D() {};
    Kernel3D( const cv::Vec3i& n_size );
    virtual ~Kernel3D() {}
    // getters
    const int& min_pos(int i) const
    {
        return min_p[i];
    }
    const int& max_pos(int i) const
    {
        return max_p[i];
    }
    virtual const inline T& offset_at(const int& x, const int& y, const int& z) const
    {
        return Data3D<T>::at(x+center[0], y+center[1], z+center[2]);
    }
    virtual inline T& offset_at(const int& x, const int& y, const int& z)
    {
        return Data3D<T>::at(x+center[0], y+center[1], z+center[2]);
    }

    // setters
    void reset( const cv::Vec3i& n_size );
private:
    // minimun and maximum possition
    cv::Vec3i min_p, max_p, center;
public:
    // Some Global Functions for Getting 3D Kernel's that are commonly used.
    static Kernel3D<T> GaussianFilter3D( cv::Vec3i size );
    inline static Kernel3D<T> GaussianFilter3D( int ksize )
    {
        smart_assert( typeid(T)==typeid(float) || typeid(T)==typeid(double),
                      "Can only use float or double" );
        return GaussianFilter3D( cv::Vec3i(ksize, ksize, ksize) );
    }
    inline static Kernel3D<T> dx()
    {
        smart_assert( typeid(T)==typeid(float) || typeid(T)==typeid(double),
                      "Can only use float or double" );
        Kernel3D<T> dx( cv::Vec3i(3,1,1) );
        dx.at( 0, 0, 0 ) = -0.5;
        dx.at( 2, 0, 0 ) =  0.5;
        return dx;
    }
    inline static Kernel3D<T> dy()
    {
        smart_assert( typeid(T)==typeid(float) || typeid(T)==typeid(double),
                      "Can only use float or double" );
        Kernel3D<T> dy( cv::Vec3i(1,3,1) );
        dy.at( 0, 0, 0 ) = -0.5;
        dy.at( 0, 2, 0 ) =  0.5;
        return dy;
    }
    inline static Kernel3D<T> dz()
    {
        smart_assert( typeid(T)==typeid(float) || typeid(T)==typeid(double),
                      "Can only use float or double" );
        Kernel3D<T> dz( cv::Vec3i(1,1,3) );
        dz.at( 0, 0, 0 ) = -0.5;
        dz.at( 0, 0, 2 ) =  0.5;
        return dz;
    }

};
