#include <iostream>
#include "detector.h"
#include "data3D.h"
#include "processing.h"

using namespace std;
using namespace cv;

extern "C" {
void vesselness(
    float* array, const int width, const int height, const int depth,
    float* response,
    float sigma_from = 1.0f,
    float sigma_to   = 8.10f,
    float sigma_step = 0.3f,
    float alpha = 1.0e-1f,
    float beta  = 5.0e0f,
    float gamma = 3.5e5f )
{
    // Sigma: Parameters for Vesselness
    // [sigma_from, sigma_to]: the potential size rang of the vessels
    // sigma_step: precision of computation
    // Parameters for vesselness, please refer to Frangi's paper
    // or this [blog](http://yzhong.co/?p=351)

    Data3D<float> im_short(width, height, depth);
    im_short.SetFromArray(array);

    Data3D<Vesselness_Sig> vn_sig;
    VesselDetector::compute_vesselness( im_short, vn_sig, sigma_from, sigma_to, sigma_step,
                                        alpha, beta, gamma );

//     Data3D<Vesselness_Sig> vn_sig_nms;
//     IP::non_max_suppress( vn_sig, vn_sig_nms );
//
//     Data3D<Vesselness_Sig> vn_sig_et;
//     IP::edge_tracing( vn_sig_nms, vn_sig_et, 0.45f, 0.05f );

    int x,y,z;
    float* array_cursor = response;
    for( z=0; z<vn_sig.get_size_z(); z++ ) {
        for( y=0; y<vn_sig.get_size_y(); y++ ) {
            for( x=0; x<vn_sig.get_size_x(); x++ ) {
                *array_cursor = vn_sig.at(x,y,z).rsp;
                array_cursor++;
            }
        }
    }
//     vn_sig_et.CopyToArray(array);
//     for (int idx=0; idx < 20; idx++) {
//         cout << array[idx] << endl;
//     }
}
}

int main(void)
{
    float* array = new float[100*100*100];
    float* response = new float[100*100*100];

    vesselness( array, 100,100,100, response );

    return 0;
}
